@@ -10,14 +10,14 @@ static PyObject *RSRFile_open(RSRFile *self, PyObject *args, PyObject *kwargs)
 {
     char *path_to_file;
     char *mode = NULL;
-    char *encoding = cp1251;
+    char *encoding = (char*)cp1251;
 
     static char *kwlist[] = {"filepath", "mode", "encoding", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s|ss", kwlist,
-        &path_to_file, &mode, &encoding))
+                                     &path_to_file, &mode, &encoding))
     {
-        PyErr_SetString(PyExc_ValueError, "Error!");
+        PyErr_SetString(PyExc_ValueError, "Wrong parameters");
         return NULL;
     }
 
@@ -37,7 +37,8 @@ static PyObject *RSRFile_open(RSRFile *self, PyObject *args, PyObject *kwargs)
             m_mode = PROT_READ | PROT_WRITE;
             break;
         default:
-            PyErr_SetString(PyExc_ValueError, "Error open mode value!");
+            PyErr_Format(PyExc_RuntimeError,
+                         "Undefine access mode value '%s'", mode);
             return NULL;
         }
     }
@@ -45,21 +46,21 @@ static PyObject *RSRFile_open(RSRFile *self, PyObject *args, PyObject *kwargs)
     int fp;
     if ((fp = open(path_to_file, f_mode)) == -1)
     {
-        PyErr_SetString(PyExc_FileNotFoundError, "Error!");
+        PyErr_Format(PyExc_FileNotFoundError, "File '%s' not found", path_to_file);
         return NULL;
     }
 
     struct stat fileInfo;
     if (stat(path_to_file, &fileInfo) == -1)
     {
-        PyErr_SetString(PyExc_FileNotFoundError, "Error!");
+        PyErr_SetString(PyExc_FileNotFoundError, "Can't read file stat");
         return NULL;
     }
     // PROT_READ | PROT_WRITE
     uint8_t *mapped = mmap(0, fileInfo.st_size, m_mode, MAP_PRIVATE, fp, 0);
     if (mapped == MAP_FAILED)
     {
-        PyErr_SetString(PyExc_MemoryError, "mapped() error!");
+        PyErr_SetString(PyExc_MemoryError, "Can't map the file");
         return NULL;
     }
     if (f_mode == O_RDONLY)
@@ -73,14 +74,13 @@ static PyObject *RSRFile_open(RSRFile *self, PyObject *args, PyObject *kwargs)
     self->file_size = fileInfo.st_size;
     self->mapped = mapped;
     self->encoding = encoding;
-    //self->encoding = cp1251;
 
     Py_RETURN_NONE;
 }
 
 static PyObject *RSRFile_close(RSRFile *self)
 {
-    if (self->fp != NULL)
+    if (self->fp != -1)
     {
         close(self->fp);
     }
@@ -88,7 +88,7 @@ static PyObject *RSRFile_close(RSRFile *self)
     {
         if (munmap(self->mapped, self->file_size) == -1)
         {
-            PyErr_SetString(PyExc_MemoryError, "munmap() error!");
+            PyErr_SetString(PyExc_MemoryError, "Can't unmap file");
             return NULL;
         }
     }
@@ -116,8 +116,6 @@ static void RSRFile_dealloc(RSRFile *self)
     CHECK_NULL_DECREF(self->EGImpTable);
     CHECK_NULL_DECREF(self->pdf);
     CHECK_NULL_DECREF(self->cdf);
-    CHECK_NULL_DECREF(self->mcs);
-    CHECK_NULL_DECREF(self->mod_mcs);
     CHECK_NULL_DECREF(self->Events);
     Py_TYPE(self)->tp_free((PyObject *)self);
 }
@@ -141,9 +139,9 @@ static PyObject *RSRFile_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
         self->EGImpTable = NULL;
         self->pdf = NULL;
         self->cdf = NULL;
-        self->mcs = NULL;
-        self->mod_mcs = NULL;
         self->Events = NULL;
+
+        self->fp = -1;
     }
     return (PyObject *)self;
 }
@@ -221,7 +219,6 @@ static PyObject *unc_summary_get(RSRFile *self, void *closure)
             PyObject *const result = create_UNCSummary(unc_struct);
             if (result == NULL)
             {
-
                 Py_RETURN_NONE;
             }
             self->UNCSummary = result;
@@ -253,7 +250,7 @@ static PyObject *timedep_summary_get(RSRFile *self, void *closure)
         {
             Py_RETURN_NONE;
         }
-        }
+    }
     Py_INCREF(self->TimeDepSummary);
     return self->TimeDepSummary;
 }
@@ -349,7 +346,6 @@ static PyObject *unc_get(RSRFile *self, const uint_fast8_t type)
     const uint8_t count = self->headers[type].Record;
     if (count > 0)
     {
-
         PyObject *result = PyTuple_New(count + 1);
 
         PyObject *header_obj = PyTuple_New(2);
@@ -559,69 +555,102 @@ static PyObject *eg_im_get(RSRFile *self, void *closure)
     return self->EGImpTable;
 }
 
-static PyObject *mcs_get(RSRFile *self, void *closure)
+static PyObject *mcs_get(RSRFile *self, PyObject *args, PyObject *kwargs)
 {
-    if (self->mcs == NULL)
+    char *kwlist[] = {"start", "end", "with_header", "mod_expand", NULL};
+
+    uint_fast32_t start = 1;
+    uint_fast32_t end = 0;
+    int with_header = 1;
+    int mod_expand = 0;
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|IIpp", kwlist,
+                                     &start, &end, &with_header, &mod_expand))
     {
-        const uint_fast32_t count = self->headers[MCSSTRUCT_OFFSET].Record;
-        if (count > 0)
+        PyErr_SetString(PyExc_ValueError, "Incorrect parameters");
+        return NULL;
+    }
+
+    const uint_fast32_t count = self->headers[MCSSTRUCT_OFFSET].Record;
+
+    if (count > 0)
+    {
+        if ((end == 0) || (end > count))
         {
-
-            PyObject *result = create_mcs(
-                (const MCSStruct *const)&self->mapped[self->headers[MCSSTRUCT_OFFSET].StartByte],
-                (const uint32_t *const)&self->mapped[self->headers[MCSEVENT_OFFSET].StartByte],
-                (const EventStruct *const)&self->mapped[self->headers[EVENT_OFFSET].StartByte],
-                (const BEEventStruct *const)&self->mapped[self->headers[BEVENT_OFFSET].StartByte],
-                (const CCFEventStruct *const)&self->mapped[self->headers[CCFEVENT_OFFSET].StartByte],
-                (const MODEventStruct *const)&self->mapped[self->headers[MODEVENT_OFFSET].StartByte],
-                count,
-                self->encoding);
-
-            if (result == NULL)
-            {
-                Py_RETURN_NONE;
-            }
-            self->mcs = result;
+            end = count;
         }
-        else
+        if (start > end)
+        {
+            PyErr_SetString(PyExc_ValueError, "End value must be more than start value");
+            return NULL;
+        }
+
+        PyObject *result = create_mcs(
+            (const MCSStruct *const)&self->mapped[self->headers[MCSSTRUCT_OFFSET].StartByte],
+            (const uint32_t *const)&self->mapped[self->headers[MCSEVENT_OFFSET].StartByte],
+            (const EventStruct *const)&self->mapped[self->headers[EVENT_OFFSET].StartByte],
+            (const BEEventStruct *const)&self->mapped[self->headers[BEVENT_OFFSET].StartByte],
+            (const CCFEventStruct *const)&self->mapped[self->headers[CCFEVENT_OFFSET].StartByte],
+            (const MODEventStruct *const)&self->mapped[self->headers[MODEVENT_OFFSET].StartByte],
+            (const uint32_t *const)&self->mapped[self->headers[7].StartByte],
+            self->encoding, start, end, with_header, mod_expand);
+
+        if (result == NULL)
         {
             Py_RETURN_NONE;
         }
+        Py_INCREF(result);
+        return result;
     }
-    Py_INCREF(self->mcs);
-    return self->mcs;
+    Py_RETURN_NONE;
 }
 
-static PyObject *mod_mcs_get(RSRFile *self, void *closure)
+static PyObject *mod_mcs_get(RSRFile *self, PyObject *args, PyObject *kwargs)
 {
-    if (self->mod_mcs == NULL)
-    {
-        const uint_fast32_t count = self->headers[MODMCSSTRUCT_OFFSET].Record;
-        if (count > 0)
-        {
-            PyObject *result = create_mcs(
-                (const MCSStruct *const)&self->mapped[self->headers[MODMCSSTRUCT_OFFSET].StartByte],
-                (const int32_t *const)&self->mapped[self->headers[MODMCSEVENT_OFFSET].StartByte],
-                (const EventStruct *const)&self->mapped[self->headers[EVENT_OFFSET].StartByte],
-                (const BEEventStruct *const)&self->mapped[self->headers[BEVENT_OFFSET].StartByte],
-                (const CCFEventStruct *const)&self->mapped[self->headers[CCFEVENT_OFFSET].StartByte],
-                (const MODEventStruct *const)&self->mapped[self->headers[MODEVENT_OFFSET].StartByte],
-                count,
-                self->encoding );
+    char *kwlist[] = {"start", "end", "with_header", NULL};
 
-            if (result == NULL)
-            {
-                Py_RETURN_NONE;
-            }
-            self->mod_mcs = result;
+    uint_fast32_t start = 1;
+    uint_fast32_t end = 0;
+    int with_header = 1;
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|IIp", kwlist,
+                                     &start, &end, &with_header))
+    {
+        PyErr_SetString(PyExc_ValueError, "Incorrect parameters");
+        return NULL;
+    }
+
+    const uint_fast32_t count = self->headers[MODMCSSTRUCT_OFFSET].Record;
+
+    if (count > 0)
+    {
+        if ((end == 0) || (end > count))
+        {
+            end = count;
         }
-        else
+        if (start > end)
+        {
+            PyErr_SetString(PyExc_ValueError, "End value must be more than start value");
+            return NULL;
+        }
+        PyObject *result = create_mcs(
+            (const MCSStruct *const)&self->mapped[self->headers[MODMCSSTRUCT_OFFSET].StartByte],
+            (const int32_t *const)&self->mapped[self->headers[MODMCSEVENT_OFFSET].StartByte],
+            (const EventStruct *const)&self->mapped[self->headers[EVENT_OFFSET].StartByte],
+            (const BEEventStruct *const)&self->mapped[self->headers[BEVENT_OFFSET].StartByte],
+            (const CCFEventStruct *const)&self->mapped[self->headers[CCFEVENT_OFFSET].StartByte],
+            (const MODEventStruct *const)&self->mapped[self->headers[MODEVENT_OFFSET].StartByte],
+            (const uint32_t *const)&self->mapped[self->headers[7].StartByte],
+            self->encoding, start, end, with_header, 0);
+
+        if (result == NULL)
         {
             Py_RETURN_NONE;
         }
+        Py_INCREF(result);
+        return result;
     }
-    Py_INCREF(self->mod_mcs);
-    return self->mod_mcs;
+    Py_RETURN_NONE;
 }
 
 static PyObject *events_get(RSRFile *self, void *closure)
@@ -668,7 +697,11 @@ static PyObject *events_get(RSRFile *self, void *closure)
                     name = modevent_struct[event_index].Name;
                     break;
                 default:
-                    PyErr_SetString(PyExc_Exception, "Error. Can't read event id. Undefine event type");
+                    PyErr_Format(PyExc_RuntimeError,
+                                 "Can't read event, undefine event type '%u' in (%u)",
+                                 event_type, i);
+                    Py_DECREF(dict_obj);
+                    Py_DECREF(value_obj);
                     return NULL;
                 }
 
@@ -739,13 +772,13 @@ static PyGetSetDef RSRFile_getsets[] = {
      "Cumulative distribution function of results", /* doc */
      NULL /* closure */},
 
-    {"mcs", (getter)mcs_get, NULL,
-     "Minimal cut sets", /* doc */
-     NULL /* closure */},
+    //{"mcs", (getter)mcs_get, NULL,
+    // "Minimal cut sets", /* doc */
+    // NULL /* closure */},
 
-    {"mod_mcs", (getter)mod_mcs_get, NULL,
-     "Mod. minimal cut sets", /* doc */
-     NULL /* closure */},
+    // {"mod_mcs", (getter)mod_mcs_get, NULL,
+    //  "Mod. minimal cut sets", /* doc */
+    //  NULL /* closure */},
 
     {"events", (getter)events_get, NULL,
      "Dictionary of events used in the minimum sections", /* doc */
@@ -758,6 +791,8 @@ static PyMethodDef RSRFile_methods[] = {
     {"close", (PyCFunction)RSRFile_close, METH_NOARGS, "Close file"},
     {"__enter__", (PyCFunction)RSRFile_enter, METH_NOARGS, "Enter the runtime context"},
     {"__exit__", (PyCFunction)RSRFile_exit, METH_VARARGS, "Exit the runtime context"},
+    {"mcs", (PyCFunction)mcs_get, METH_VARARGS | METH_KEYWORDS, "Get minimal cut sets"},
+    {"mod_mcs", (PyCFunction)mod_mcs_get, METH_VARARGS | METH_KEYWORDS, "Get mod. minimal cut sets"},
     {NULL} /* Sentinel */
 };
 
